/*                 _,.---._    .-._        .--.-. ,--.--------.         
 *   _,..---._   ,-.' , -  `. /==/ \  .-._/==/  //==/,  -   , -\
 * /==/,   -  \ /==/_,  ,  - \|==|, \/ /, |==\ -\\==\.-.  - ,-./        
 * |==|   _   _\==|   .=.     |==|-  \|  | \==\- \`--`\==\- \
 * |==|  .=.   |==|_ : ;=:  - |==| ,  | -|  `--`-'     \==\_ \
 * |==|,|   | -|==| , '='     |==| -   _ |             |==|- |          
 * |==|  '='   /\==\ -    ,_ /|==|  /\ , |             |==|, |          
 * |==|-,   _`/  '.='. -   .' /==/, | |- |             /==/ -/          
 * `-.`.____.'     `--`--''   `--`./  `--`             `--`--`          
 *                    _ __    ,---.      .-._         .=-.-.  _,.----.   
 *                 .-`.' ,`..--.'  \    /==/ \  .-._ /==/_ /.' .' -   \
 *                /==/, -   \==\-/\ \   |==|, \/ /, /==|, |/==/  ,  ,-'  
 *               |==| _ .=. /==/-|_\ |  |==|-  \|  ||==|  ||==|-   |  .  
 *               |==| , '=',\==\,   - \ |==| ,  | -||==|- ||==|_   `-' \
 *               |==|-  '..'/==/ -   ,| |==| -   _ ||==| ,||==|   _  , | 
 *               |==|,  |  /==/-  /\ - \|==|  /\ , ||==|- |\==\.       / 
 *               /==/ - |  \==\ _.\=\.-'/==/, | |- |/==/. / `-.`.___.-'  
 *               `--`---'   `--`        `--`./  `--``--`-`
 *
 *	@(#)Copyright (c) 2023, Nathan D. Fisher.
 *
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice
 *      are preserved in all copies and that due credit be given
 *      to the author.
 *      2/ that any changes to this code are clearly commented
 *      as such so that the author does not get blamed for bugs
 *      other than his own.
*/

#ifndef HAGGIS_H
#define HAGGIS_H

#include <stdint.h>

typedef uint64_t u64;
typedef uint32_t u32;
typedef uint16_t u16;
typedef uint8_t u8;

u64 le_bytes_to_u64(u8 arr[8]);
u32 le_bytes_to_u32(u8 arr[4]);
u16 le_bytes_to_u16(u8 arr[2]);
void u64_to_le_bytes(u64 num, u8 arr[8]);
void u32_to_le_bytes(u32 num, u8 arr[4]);
void u16_to_le_bytes(u16 num, u8 arr[2]);

struct haggis_device {
    uint32_t major;
    uint32_t minor;
};

enum haggis_algorithm {
    md5,
    sha1,
    sha256,
    skip,
};

union haggis_sum {
    char md5[16];
    char sha1[20];
    char sha256[32];
};

struct haggis_checksum {
    enum haggis_algorithm tag;
    union haggis_sum *sum;
};

struct haggis_normal {
    uint64_t len;
    struct haggis_checksum *cksum;
    unsigned char *data[];
};

enum haggis_typeflag {
    normal,
    hardlink,
    softlink,
    directory,
    character,
    block,
    fifo,
    eof,
};

union haggis_ft {
    struct haggis_normal *file;
    char target[4096];
    struct haggis_device *dev;
};

struct haggis_filetype {
    enum haggis_typeflag tag;
    union haggis_ft *f_type;
};

struct haggis_node {
    uint32_t uid;
    uint32_t gid;
    uint64_t mtime;
    uint16_t mode;
    struct haggis_filetype *filetype;
    char *name[];
};

#endif // !HAGGIS_H
